<?php

namespace RonPortal\Controls;

use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;

/**
 * Vychozi filtr na obdobi
 */
class ObdobiFilter extends FilterFormFilter
{
    /** @var RonPortalFilterForm */
    protected $filterForm;

    /** @var ComboBoxObdobi */
    protected $comboObdobi;

    /** @var bool */
    public $jenOtevreneObdobi = false;

    /** @var int    Maximalni pocet zobrazenych zaznamu */
    public $maximalniPocetObdobi = 0;

    /** @var bool       Urcuje, jestli je filtr obdobi zapnuty */
    public $jenObdobi = false;

    /** @var int|null   Konec zvoleneho obdobi */
    public $konecObdobi = null;

    public function __construct(RonPortalFilterForm $filterForm, string $name, string $type = self::TYPE_LINKS)
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);

        $this ->filterForm = $filterForm;
        $this ->rowId = 'filterrow_obdobi';

        $this ->caption = TextManager::_('Obdobi');
        $this ->sessionName = 'filtr' . $this ->name; // pro kompatibilitu

        $this ->maximalniPocetObdobi = RonPortalSettings::getIntValue('maximalnipocetobdobi');

        $this ->htmlAttributes = [
            'id' => 'adsfilterform_obdobi',
            'data-ron-autosubmit' => $this ->filterForm ->autoSubmit ? '1' : '0'
        ];
    }

    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'jenOtevreneObdobi' :
            case 'maximalniPocetObdobi' :
                $this ->{$key} = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    public function load(Database $database)
    {
        if ($this ->filterForm ->obdobiPovinne) {
            $this ->jenObdobi = true;
        } else if (isset($_POST['adsfilterform'])) {
            if ($this ->filterForm ->obdobiNeboDatumPovinne) {
                $this ->jenObdobi = (int)($_POST['jen' . $this ->name] ?? 1) != 2;
                $_SESSION['filtrobdobinebodatum'] = $this ->jenObdobi ? 1 : 2;
            } else {
                $this->jenObdobi = isset($_POST['jen' . $this ->name]);
                $_SESSION['filtrjen' . $this ->name] = $this->jenObdobi;
            }

        } else if (isset($_GET[$this ->name])) {
            if ($this ->filterForm ->obdobiNeboDatumPovinne) {
                $this ->jenObdobi = (int)($_GET['jenobdobi'] ?? 1) != 2;
                $_SESSION['filtrobdobinebodatum'] = $this ->jenObdobi ? 1 : 2;
            } else {
                $this->jenObdobi = (int)($_GET['jenobdobi'] ?? 0);
                $_SESSION['filtrjen' . $this ->name] = $this->jenObdobi;
            }
        } else {
            $this ->jenObdobi = $this ->filterForm ->obdobiNeboDatumPovinne
                ? ($_SESSION['filtrobdobinebodatum'] ?? 1) != 2
                : $_SESSION['filtrjen' . $this ->name] ?? false;
        }

        $obdobi = RonPortalSession::obdobi();

        try {
            if (isset($_POST[$this ->name])) {
                $obdobi = DateUtils::urlStringToDate($_POST[$this ->name], true);  //(int)$_POST['obdobi'];
            }
            else if (isset($_GET[$this ->name])) {
                $obdobi = DateUtils::urlStringToDate($_GET[$this ->name], true); // (int)$_GET['obdobi'];
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        $this ->comboObdobi = new ComboBoxObdobi($this ->name);
        $this ->comboObdobi
            ->setClassName('form-select adsfilterforminput_full ' . $this ->addClass)
            ->setHtmlParams($this ->htmlAttributes)
            ->setOrderBy($this ->optionsSqlOrder ?? ' PocatekObdobi DESC')
            ->setSqlFilter($this ->optionsSqlFilter);

        $this ->comboObdobi ->maximalniPocetObdobi = $this ->maximalniPocetObdobi;

        $this ->comboObdobi ->loadObdobi($database,
            $this->filterForm ->obdobiAplikace, false,
            $this->jenOtevreneObdobi, false,
            $obdobi);

        $this ->value = $this ->comboObdobi ->pocatekObdobi;
        $this ->konecObdobi = $this ->comboObdobi ->konecObdobi;
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case 'datumOd' :
                return $this ->value;

            case 'konecObdobi' :
            case 'datumDo' :
                return $this ->konecObdobi;

            case 'uzavreno' :
                return $this ->comboObdobi
                    ? $this ->comboObdobi ->uzavreno
                    : null;

            case 'popis' :
                return $this ->comboObdobi
                    ? $this ->comboObdobi ->popisObdobi
                    : null;

            case 'jenObdobi' :
                return $this ->jenObdobi;
        }

        return parent::getValue($specification);
    }


    public function renderCaption()
    {
        if ($this->filterForm ->obdobiPovinne) {
            echo $this ->caption;
        } else {
            GuiUtils::renderCheckbox('jenobdobi',
                $this ->caption, $this ->jenObdobi,
                [
                    'data-excluded-checkboxes' => 'input[name=jendatumoddo]',
                    'type' => $this ->filterForm ->obdobiNeboDatumPovinne ? 'radio' : 'checkbox'
                ],
                'clickAutoSubmit', 1, true);
        }
    }


    public function render()
    {
        $this ->comboObdobi ->render();

        // tlacitka pro posun na dalsi a predchozi obdobi
        echo
            '<span id="adsfilterform_obdobi_fast" class="desktop-only">' .
            '<i class="inline-glyph glyph-button fa glyph-left comboBoxMove" data-submit="1" data-selector="#adsfilterform_obdobi"></i>' .
            '<i class="inline-glyph glyph-button fa glyph-right comboBoxMove" data-step="-1" data-submit="1" data-selector="#adsfilterform_obdobi"></i>' .
            '</span>';

        // zobrazi zamek, pokud je obdobi uzavrene
        if ($this ->comboObdobi->uzavreno) {
            echo '<span class="inline-glyph fa glyph-lock"></span>';
        }
    }
}



/**
 * Obecny filtr na rozdeleni. Ve formulari muze byt pouzit vicekrat (pro ruzny typ rozdeleni)
 */
class RozdeleniFilter extends FilterFormFilter
{
    /** @var RonPortalFilterForm */
    protected $filterForm;

    /** @var ComboBoxes    Comboboxy pro vyber rozdeleni (podle popisu a kodu) */
    protected $comboBoxesRozdeleni;

    /** @var int    Kod typu rozdeleni */
    public $typRozdeleni;

    /** @var ?PravaNaOsoby  Typ prava, podle ktereho se seznam rozdeleni muze filtrovat */
    public $prava;

    /**
     * @var int Urcuje, jestli se nactou jen vlastni rozdeleni
     *          0=vse, 1=jen vlastni, 2=podle prav (prava mohou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene
     */
    public $jenVlastniRozdeleni = 0;


    public function __construct(RonPortalFilterForm $filterForm, string $name, string $type = self::TYPE_LINKS)
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);

        $this ->filterForm = $filterForm;
    }



    /**
     * @param Option $option1
     * @param Option $option2
     * @return int
     */
    protected static function compareOptionsRozdeleni($option1, $option2)
    {
        if (!$option1) {
            return $option2 ? -1 : 0;
        }

        if (!$option2) {
            return 1;
        }

        if ($option1 ->key < $option2 ->key) {
            return -1;
        }
        if ($option1 ->key > $option2 ->key) {
            return 1;
        }

        return 0;
    }


    /**
     * @param string $htmlKod
     * @param int $typRozdeleni
     * @param bool $jenVlastni
     * @param $filtr
     * @param $value
     * @return ComboBoxes
     * @throws Exception
     */
    protected function loadDataRozdeleni(Database $database, $htmlKod)
    {
        if ($this->filterForm ->showObdobi || $this->filterForm ->showVariantni) {
            $datumOd = $this->filterForm ->vybraneObdobiOd;
            $datumDo = $this->filterForm ->vybraneObdobiDo;
        } else if ($this->filterForm ->showDatumOdDo && ($this->filterForm ->datumOdDoPovinne || $this ->filterForm ->filtrovatDatum)) {
            $datumOd = $this ->filterForm ->datumOd;
            $datumDo = $this ->filterForm ->datumDo;
        } else if ($this->filterForm ->showDatum) {
            $datumOd = $this ->filterForm ->datum;
            $datumDo = $datumOd;
        } else {
            $datumOd = time();
            $datumDo = null;
        }

        if (($this ->jenVlastniRozdeleni === 2 || $this ->jenVlastniRozdeleni === 3)  &&  $this ->prava) {  // $this ->filterForm ->f_pravarozdeleni
            $sql = $this ->prava ->sqlRozdeleni($database,
                'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass',
                (int)$this ->typRozdeleni, $datumOd, $datumDo, $this ->jenVlastniRozdeleni == 2);
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                ' AND (DatumZruseni >= ' . $database ->dateToString($datumOd) . ' OR DatumZruseni IS NULL) ';

            if ($datumDo)
                $sql .=
                    ' AND DatumVytvoreni <= ' . $database ->dateToString($datumDo);

            $filtrRozdeleni = ($this  ->jenVlastniRozdeleni === 1)
                ? $this->sestavFiltrRozdeleni(1)  // vcetne podrizenych
                : $this->filtrRozdeleniPodlePrav($this ->typRozdeleni);

            if ($filtrRozdeleni) {
                $sql .= ' AND (' . $filtrRozdeleni . ')';
            }
        }

        if ($this ->optionsSqlFilter) {
            if ($sql instanceof SqlStatement) {
                $sql ->append(' AND (' . $this ->optionsSqlFilter . ')');
            } else {
                $sql .= ' AND (' . $this ->optionsSqlFilter . ')';
            }
        }

        if (isset($_GET[$this ->name])) {  // if ($this ->useGet  && isset($_GET[$this ->name]))
            // pokud pouziva GET, tak povoli v URL vybrat i rozdeleni, do ktereho vybrana osoba nepatri
            // (jinak by neslo prepnout rozdeleni)
            // POZOR! V URL pak muze byt jina osoba nez skutecne nactena
        } else if (isset($_GET['osoba'])) {
            if (($this ->value ?? '') !== '' ) {
                // pokud je nastavena osoba v URL adrese, tak zkontroluje, jestli tato osoba patri do aktualne vybraneho rozdeleni
                // a pokud ne, tak rozdeleni vypne - je to nutne, aby fungovaly napr. odkazy v emailech
                if (!Rozdeleni::jeVRozdeleni($database, null, $_GET['osoba'], $this ->typRozdeleni, $this ->value, 3, $datumOd, $datumDo)) { // 3 = vcetne podrizenych
                    $this ->value = null;
                }
            }
        }

        MessageManager::addDebugLog('Vyber rozdeleni ' . $this ->typRozdeleni . ':');
        MessageManager::addDebugLog($sql);

        // vyber rozdeleni podle popisu
        $combo1 = new ComboBox($this ->name);
        $combo1
            ->setClassName('form-control adsfilterforminput_popis ronselect')
            ->setCssClassIndex(2)
            ->setShowCode(true)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setTranslationPrefix("db_rozdeleni_{$this ->typRozdeleni}_popis_")
            ->setHtmlParams([
                'data-kod-typu-rozdeleni' => $this ->typRozdeleni,
                'data-ron-select2' => "#ronportalfilter_{$this ->name}_kod",
                'data-ronselect-class' => "ronselect adsfilterforminput_popis",
                'data-ron-autosubmit' => ($this ->filterForm ->autoSubmit ? '1' : '0'),
                'data-insert-position' => 'before',
                'readonly' => 'readonly'
            ]);

        $sqlPopis = $sql instanceof SqlStatement
            ? new SqlStatement($sql ->sqlText . ' ORDER BY 2', $sql ->parameters)
            : $sql . ' ORDER BY Rozdeleni.Popis';
        $combo1->loadData($database, $sqlPopis, $this ->value);

//        if (isset($combo1->f_selectedvalue)) {
//            $result = $combo1->f_selectedvalue;
//        }
        $options2 = [];
        foreach ($combo1 ->data as $option) {
            $option->params['data-popis'] = $option ->caption;
            $options2[] = $option;
        }

        usort($options2, 'RonPortal\Controls\RozdeleniFilter::compareOptionsRozdeleni');

        // vyber kodu rozdeleni
        $combo2 = new ComboBox(null); // $htmlKod . '_kod'
        $combo2->className = 'form-control adsfilterforminput_kod desktop-only ronselect';
        $combo2->cssClassIndex = 2;
        $combo2->captionIndex = 0; // kod - kod
        $combo2->emptyRow = TextManager::_('_vse_');
//        $sqlKod = $sql instanceof SqlStatement
//            ? new SqlStatement($sql ->sqlText . ' ORDER BY Rozdeleni.Kod', $sql ->parameters)
//            : $sql . ' ORDER BY Rozdeleni.Kod';
//        $combo2->loadData($this ->database, $sqlKod, $value);

        $combo2 ->valueFieldName = 'key';
        $combo2 ->keyFieldName = 'key';
        $combo2 ->titleFieldName = 'caption';
        $combo2 ->cssClassFieldName = 'cssClass';

        $combo2 ->addOptions($options2);

        $combo2->setSelectedOption($this ->value);

        $combo2->htmlParams =
            "data-ron-select2='{$this ->name}'" . ($this ->filterForm ->autoSubmit ? ' data-ron-autosubmit="1"' : '') .
            ' data-ronselect-class="ronselect adsfilterforminput_kod"' .
            ' data-columns="value;popis"' .
            ' data-drop-down-align="right"' .
            " id='ronportalfilter_{$htmlKod}_kod'";

        return new ComboBoxes($combo1, $combo2);
    }

    public function load(Database $database)
    {
        $typRozdeleniSuffix = $this ->vlastniTypRozdeleni ? $this ->kodTypuRozdeleni : null;

        // nacte rozdeleni zadane uzivatelem
        $this ->setUserSettingsItem('ads_rozdeleni' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''), 'rozdeleni', true, true);
        $this ->vcetnePodrizenychRozdeleni = (int)$this ->setUserSettingsItem('ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
            'rozdeleni_vcetne_podrizenych', true, true);
//        $this ->vcetnePodrizenychRozdeleni = $this ->setUserSettingsItemCheckBox(
//            'ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleniSuffix ? '_' . $typRozdeleniSuffix : ''),
//            'rozdeleni_vcetne_podrizenych', 'rozdeleni',
//            true, true, $this->vcetnePodrizenychRozdeleni);

        if ($this->pouzitPopisTypuRozdeleni) {
            $this->popisTypuRozdeleni = $this->loadPopisTypuRozdeleni($this->kodTypuRozdeleni);
        }

        $this ->comboBoxesRozdeleni = $this ->loadDataRozdeleni('rozdeleni',
            $this->kodTypuRozdeleni, $this->jenVlastniRozdeleni, $this->filtrRozdeleni,
            RonPortalSession::rozdeleni($typRozdeleniSuffix));

        if (isset($this ->comboBoxesRozdeleni ->combo1 ->selectedOption)) {
            $this ->kodRozdeleni = $this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key;
            RonPortalSession::setRozdeleni($this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
        } else {
            $this ->kodRozdeleni = null;
        }
    }


}
